@@ -30,7 +30,7 @@
  * worn off your battery is.
  *
  */
-void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char *path, const char *format, int low_threshold, char *threshold_type, bool last_full_capacity, bool integer_battery_capacity) {
+void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char *path, const char *format, const char *format_down, int low_threshold, char *threshold_type, bool last_full_capacity, bool integer_battery_capacity) {
         time_t empty_time;
         struct tm *empty_tm;
         char buf[1024];
@@ -55,13 +55,13 @@ void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char
         memset(emptytimebuf, '\0', sizeof(emptytimebuf));
         memset(consumptionbuf, '\0', sizeof(consumptionbuf));
 
-        INSTANCE(path);
-
-#if defined(LINUX)
         static char batpath[512];
         sprintf(batpath, path, number);
+        INSTANCE(batpath);
+
+#if defined(LINUX)
         if (!slurp(batpath, buf, sizeof(buf))) {
-                OUTPUT_FULL_TEXT("No battery");
+                OUTPUT_FULL_TEXT(format_down);
                 return;
         }
 
@@ -123,7 +123,7 @@ void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char
         }
 
         if ((full_design == -1) || (remaining == -1)) {
-                OUTPUT_FULL_TEXT("No battery");
+                OUTPUT_FULL_TEXT(format_down);
                 return;
         }
 
@@ -196,19 +196,19 @@ void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char
         size_t sysctl_size = sizeof(sysctl_rslt);
 
         if (sysctlbyname(BATT_LIFE, &sysctl_rslt, &sysctl_size, NULL, 0) != 0) {
-                OUTPUT_FULL_TEXT("No battery");
+                OUTPUT_FULL_TEXT(format_down);
                 return;
         }
 
         present_rate = sysctl_rslt;
         if (sysctlbyname(BATT_TIME, &sysctl_rslt, &sysctl_size, NULL, 0) != 0) {
-                OUTPUT_FULL_TEXT("No battery");
+                OUTPUT_FULL_TEXT(format_down);
                 return;
         }
 
         remaining = sysctl_rslt;
         if (sysctlbyname(BATT_STATE, &sysctl_rslt, &sysctl_size, NULL,0) != 0) {
-                OUTPUT_FULL_TEXT("No battery");
+                OUTPUT_FULL_TEXT(format_down);
                 return;
         }
 
@@ -234,6 +234,15 @@ void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char
                 minutes -= (hours * 60);
                 (void)snprintf(remainingbuf, sizeof(remainingbuf), "%02dh%02d",
                                max(hours, 0), max(minutes, 0));
+		if (strncmp(threshold_type, "percentage", strlen(threshold_type)) == 0
+		    && present_rate < low_threshold) {
+			START_COLOR("color_bad");
+			colorful_output = true;
+		} else if (strncmp(threshold_type, "time", strlen(threshold_type)) == 0
+			   && remaining < (u_int) low_threshold) {
+			START_COLOR("color_bad");
+			colorful_output = true;
+		}
         }
 #elif defined(__OpenBSD__)
 	/*
@@ -257,7 +266,7 @@ void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char
 	/* Don't bother to go further if there's no battery present. */
 	if ((apm_info.battery_state == APM_BATTERY_ABSENT) ||
 	    (apm_info.battery_state == APM_BATT_UNKNOWN)) {
-		OUTPUT_FULL_TEXT("No battery");
+		OUTPUT_FULL_TEXT(format_down);
 		return;
 	}
 
