@@ -20,6 +20,13 @@
 #include <sys/resource.h>
 #endif
 
+#if defined(__NetBSD__)
+#include <sys/param.h>
+#include <sys/resource.h>
+#include <sys/sysctl.h>
+#include <sys/sched.h>
+#endif
+
 #include "i3status.h"
 
 static int prev_total = 0;
@@ -50,9 +57,9 @@ void print_cpu_usage(yajl_gen json_gen, char *buffer, const char *format) {
         diff_usage = (diff_total ? (1000 * (diff_total - diff_idle)/diff_total + 5)/10 : 0);
         prev_total = curr_total;
         prev_idle  = curr_idle;
-#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
+#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 
-#if defined(__FreeBSD__) || defined(__DragonFly__)
+#if defined(__FreeBSD__) || defined(__DragonFly__) || defined(__NetBSD__)
         size_t size;
         long cp_time[CPUSTATES];
         size = sizeof cp_time;
